SET client_encoding = 'UTF8';
SET standard_conforming_strings = off;
SET check_function_bodies = true;
SET client_min_messages = warning;
SET escape_string_warning = off;
-- PostgreSQL membership/role requiremets

CREATE TABLE Users (
	pId										character(36)			NOT NULL,
	Username									character varying(255)	NOT NULL,
	ApplicationName							character varying(255)	NOT NULL,
	Email										character varying(128)	NULL,
	Comment									character varying(128)	NULL,
	Password									character varying(255)	NOT NULL,
	PasswordQuestion							character varying(255)	NULL,
	PasswordAnswer							character varying(255)	NULL,
	IsApproved								boolean					NULL, 
	LastActivityDate							timestamptz				NULL,
	LastLoginDate								timestamptz				NULL,
	LastPasswordChangedDate					timestamptz				NULL,
	CreationDate								timestamptz				NULL, 
	IsOnLine									boolean					NULL,
	IsLockedOut								boolean					NULL,
	LastLockedOutDate							timestamptz				NULL,
	FailedPasswordAttemptCount				integer					NULL,
	FailedPasswordAttemptWindowStart			timestamptz				NULL,
	FailedPasswordAnswerAttemptCount			integer					NULL,
	FailedPasswordAnswerAttemptWindowStart	timestamptz				NULL,
	CONSTRAINT users_pkey PRIMARY KEY (pId),
	CONSTRAINT users_username_application_unique UNIQUE (Username, ApplicationName)
);

CREATE INDEX users_email_index ON Users (Email);
CREATE INDEX users_islockedout_index ON Users (IsLockedOut);

-- PostgreSQL 8 Role Provider Schema

CREATE TABLE Roles (
	Rolename				character varying(255)	NOT NULL,
	ApplicationName		character varying(255)	NOT NULL,
	CONSTRAINT roles_pkey PRIMARY KEY (Rolename, ApplicationName)
);

CREATE TABLE UsersInRoles (
	Username				character varying(255)	NOT NULL,
	Rolename				character varying(255)	NOT NULL,
	ApplicationName		character varying(255)	NOT NULL,
	CONSTRAINT usersinroles_pkey PRIMARY KEY (Username, Rolename, ApplicationName),
	CONSTRAINT usersinroles_username_fkey FOREIGN KEY (Username, ApplicationName) REFERENCES Users (Username, ApplicationName) ON DELETE CASCADE,
	CONSTRAINT usersinroles_rolename_fkey FOREIGN KEY (Rolename, ApplicationName) REFERENCES Roles (Rolename, ApplicationName) ON DELETE CASCADE
);

-- PostgreSQL 8 Profile Provider Schema

CREATE TABLE Profiles (
	pId					character(36)			NOT NULL,
	Username				character varying(255)	NOT NULL,
	ApplicationName		character varying(255)	NOT NULL,
	IsAnonymous			boolean					NULL,
	LastActivityDate		timestamptz				NULL,
	LastUpdatedDate		timestamptz				NULL,
	CONSTRAINT profiles_pkey PRIMARY KEY (pId),
	CONSTRAINT profiles_username_application_unique UNIQUE (Username, ApplicationName),
	CONSTRAINT profiles_username_fkey FOREIGN KEY (Username, ApplicationName) REFERENCES Users (Username, ApplicationName) ON DELETE CASCADE
);

CREATE INDEX profiles_isanonymous_index ON Profiles (IsAnonymous);

CREATE TABLE ProfileData (
	pId					character(36)			NOT NULL,
	Profile				character(36)			NOT NULL,
	Name					character varying(255)	NOT NULL,
	ValueString			text					NULL,
	ValueBinary			bytea					NULL,
	CONSTRAINT profiledata_pkey PRIMARY KEY (pId),
	CONSTRAINT profiledata_profile_name_unique UNIQUE (Profile, Name),
	CONSTRAINT profiledata_profile_fkey FOREIGN KEY (Profile) REFERENCES Profiles (pId) ON DELETE CASCADE
);

-- PostgreSQL 8 Session-Store Provider Schema

CREATE TABLE Sessions (
	SessionId				character varying(80)	NOT NULL,
	ApplicationName		character varying(255)	NOT NULL,
	Created				timestamptz				NOT NULL,
	Expires				timestamptz				NOT NULL,
	Timeout				integer					NOT NULL,
	Locked				boolean					NOT NULL,
	LockId				integer					NOT NULL,
	LockDate				timestamptz				NOT NULL,
	Data					text					NULL,
	Flags					integer					NOT NULL,
	CONSTRAINT sessions_pkey PRIMARY KEY (SessionId, ApplicationName)
);

-- ported from upstream
CREATE TABLE classifieds_Ads (
	Id serial NOT NULL ,
	MemberId int NOT NULL ,
	CategoryId int NOT NULL ,
	Title varchar (100) NOT NULL ,
	Description text NOT NULL ,
	URL varchar (500) NOT NULL ,
	Price NUMERIC(19,4) NOT NULL ,
	Location varchar (50) NOT NULL ,
	ExpirationDate timestamptz NULL ,
	DateCreated timestamptz NOT NULL ,
	DateApproved timestamptz NULL ,
	NumViews int NOT NULL ,
	NumResponses int NOT NULL ,
	AdLevel int NOT NULL ,
	AdStatus int NOT NULL ,
	AdType int NOT NULL ,
	PreviewImageId int NULL 
);

CREATE TABLE classifieds_Categories (
	Id serial NOT NULL ,
	ParentCategoryId int NULL ,
	Path varchar (800) NULL ,
	Name varchar (50) NOT NULL ,
	NumActiveAds int NOT NULL 
);

CREATE TABLE classifieds_Locations (
	Id serial NOT NULL ,
	Name varchar (50) NOT NULL 
);

CREATE TABLE classifieds_Members (
	Id serial NOT NULL ,
	AspNetUsername varchar (256) NOT NULL ,
	AspNetApplicationName varchar (256) NOT NULL ,
	DateCreated timestamptz NOT NULL 
);

CREATE TABLE classifieds_Photos (
	Id serial NOT NULL ,
	AdId int NOT NULL ,
	BytesFull bytea NULL ,
	BytesSmall bytea NULL ,
	BytesMedium bytea NULL ,
	IsMainPreview boolean NOT NULL ,
	DateCreated timestamptz NOT NULL 
);

CREATE TABLE classifieds_SavedAds (
	Id serial NOT NULL ,
	MemberId int NOT NULL ,
	AdId int NOT NULL ,
	DateCreated timestamptz NOT NULL 
);


ALTER TABLE classifieds_Ads  ADD 
	CONSTRAINT PK_Ads PRIMARY KEY   
	(
		Id
	);

ALTER TABLE classifieds_Categories  ADD 
	CONSTRAINT PK_Categories PRIMARY KEY   
	(
		Id
	);

ALTER TABLE classifieds_Locations  ADD 
	CONSTRAINT PK_Locations PRIMARY KEY   
	(
		Id
	);

ALTER TABLE classifieds_Members  ADD 
	CONSTRAINT PK_Members PRIMARY KEY   
	(
		Id
	);

ALTER TABLE classifieds_Photos  ADD 
	CONSTRAINT PK_Photos PRIMARY KEY   
	(
		Id
	);

ALTER TABLE classifieds_SavedAds  ADD 
	CONSTRAINT PK_SavedAds PRIMARY KEY   
	(
		Id
	)
;

ALTER TABLE classifieds_Ads
	ALTER Price SET DEFAULT (0.0),
	ALTER DateCreated SET DEFAULT (now()),
	ALTER NumViews SET DEFAULT (0),
	ALTER NumResponses SET DEFAULT (0),
	ALTER AdLevel SET DEFAULT (0),
	ALTER AdStatus SET DEFAULT (0),
	ALTER AdType SET DEFAULT (1);

ALTER TABLE classifieds_Categories ALTER NumActiveAds SET DEFAULT (0);

ALTER TABLE classifieds_Members ALTER DateCreated SET DEFAULT (now());

ALTER TABLE classifieds_Photos
	ALTER IsMainPreview SET DEFAULT (FALSE),
	ALTER DateCreated SET DEFAULT (now());

ALTER TABLE classifieds_SavedAds ALTER DateCreated SET DEFAULT (now());

ALTER TABLE classifieds_Ads ADD 
	CONSTRAINT FK_Ads_Categories FOREIGN KEY 
	(
		CategoryId
	) REFERENCES classifieds_Categories (
		Id
	) ON DELETE CASCADE  ON UPDATE CASCADE;

ALTER TABLE classifieds_Ads ADD 
	CONSTRAINT FK_Ads_Members FOREIGN KEY 
	(
		MemberId
	) REFERENCES classifieds_Members (
		Id
	) ON DELETE CASCADE  ON UPDATE CASCADE;

ALTER TABLE classifieds_Categories ADD 
	CONSTRAINT FK_Categories_Categories FOREIGN KEY 
	(
		ParentCategoryId
	) REFERENCES classifieds_Categories (
		Id
	);

ALTER TABLE classifieds_Photos ADD 
	CONSTRAINT FK_Photos_Ads FOREIGN KEY 
	(
		AdId
	) REFERENCES classifieds_Ads (
		Id
	) ON DELETE CASCADE  ON UPDATE CASCADE;

ALTER TABLE classifieds_SavedAds ADD 
	CONSTRAINT FK_SavedAds_Ads FOREIGN KEY 
	(
		AdId
	) REFERENCES classifieds_Ads (
		Id
	) ON DELETE CASCADE  ON UPDATE CASCADE;

ALTER TABLE classifieds_SavedAds ADD 
	CONSTRAINT FK_SavedAds_Members FOREIGN KEY 
	(
		MemberId
	) REFERENCES classifieds_Members (
		Id
	);

CREATE VIEW ClassifiedsView_Ads
AS
SELECT     classifieds_Ads.Id, classifieds_Ads.MemberId, classifieds_Ads.CategoryId, classifieds_Ads.Title, classifieds_Ads.Description, classifieds_Ads.URL, classifieds_Ads.Price, classifieds_Ads.Location, 
                      classifieds_Ads.ExpirationDate, classifieds_Ads.DateCreated, classifieds_Ads.DateApproved, classifieds_Ads.NumViews, classifieds_Ads.NumResponses, classifieds_Ads.AdLevel, 
                      classifieds_Ads.AdStatus, classifieds_Ads.AdType, classifieds_Ads.PreviewImageId, classifieds_Members.AspNetUsername AS MemberName, 
                      classifieds_Categories.Name AS CategoryName
FROM         classifieds_Ads INNER JOIN
                      classifieds_Members ON classifieds_Ads.MemberId = classifieds_Members.Id INNER JOIN
                      classifieds_Categories ON classifieds_Ads.CategoryId = classifieds_Categories.Id;



CREATE VIEW PhotosView
AS
SELECT     Id, AdId, IsMainPreview, DateCreated
FROM         classifieds_Photos;


CREATE TYPE CountReturn AS (Count int);

CREATE FUNCTION CountAdResponsesByStatus (pAdStatus int) RETURNS SETOF CountReturn AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT      CAST(SUM(NumResponses) AS int) AS Count
	FROM         classifieds_Ads 
	WHERE (pAdStatus IS NULL OR AdStatus = pAdStatus) LOOP
	      RETURN NEXT r;
	END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION CountAdViewsByStatus (pAdStatus int) RETURNS SETOF CountReturn AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT     CAST(SUM(NumViews) AS int) AS Count
	FROM         classifieds_Ads
	WHERE     (pAdStatus IS NULL OR AdStatus = pAdStatus) LOOP
	      RETURN NEXT r;
	END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;


CREATE TYPE IdReturn AS (Id int);
CREATE TYPE AdCountReturn AS (AdCount int);

CREATE FUNCTION CountAdsByCategory (pCategoryId int, pAdStatus int) RETURNS SETOF AdCountReturn AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT CAST(Count(Id) AS int) AS AdCount FROM classifieds_Ads
	WHERE
	(pAdStatus = 0 OR AdStatus = pAdStatus) AND
	(pCategoryId = 0
		OR 
		CategoryId IN (
		SELECT Id FROM classifieds_Categories
		WHERE Path LIKE
			  (SELECT Path
			   FROM classifieds_Categories
			   WHERE Id = pCategoryId ) || '%')) LOOP
	    RETURN NEXT r;
        END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION CountAdsByStatus (pAdStatus int, pMinDateCreated timestamptz) RETURNS SETOF CountReturn AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT     CAST(COUNT(*) AS int) AS Count
	FROM classifieds_Ads 
	WHERE (pAdStatus IS NULL OR AdStatus = pAdStatus) AND
	(pMinDateCreated IS NULL OR DateCreated > pMinDateCreated) LOOP
	     RETURN NEXT r;
        END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION CountLocations () RETURNS SETOF CountReturn AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT     CAST(COUNT(*) AS int) AS Count
	FROM         classifieds_Locations LOOP
	     RETURN NEXT r;
	END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION CountMembersByDateRange (pStartDate timestamptz, pEndDate timestamptz) RETURNS SETOF CountReturn AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT     CAST(COUNT(*) AS int) AS Count
	FROM         classifieds_Members
	WHERE   
	(pStartDate IS NULL OR DateCreated > pStartDate) AND
	(pEndDate IS NULL OR DateCreated < pEndDate) LOOP
	     RETURN NEXT r;
	END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION CountTopCategories () RETURNS SETOF CountReturn AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT     CAST(COUNT(*) AS int) AS Count
	FROM         classifieds_Categories
	WHERE     (ParentCategoryId IS NULL) LOOP
	    RETURN NEXT r;
        END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION CreateCategory (pParentCategoryId int, pName varchar(50)) RETURNS void AS $$
BEGIN
	INSERT INTO classifieds_Categories (ParentCategoryId, Name ) VALUES (pParentCategoryId, pName);
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION ExpireAd (pAdId int, pMemberId int, pAdStatus int) RETURNS void AS $$
BEGIN
	UPDATE classifieds_Ads
	SET
		AdStatus = pAdStatus,
		ExpirationDate = now()
	WHERE
		Id = pAdId AND MemberId = pMemberId;
END;
$$ LANGUAGE plpgsql;


CREATE TYPE GetAdByIdType AS (
	Id int ,
	MemberId int ,
	CategoryId int ,
	Title varchar (100) ,
	Description text ,
	URL varchar (500) ,
	Price NUMERIC(19,4) ,
	Location varchar (50) ,
	ExpirationDate timestamptz ,
	DateCreated timestamptz ,
	DateApproved timestamptz ,
	NumViews int ,
	NumResponses int ,
	AdLevel int ,
	AdStatus int ,
	AdType int ,
	PreviewImageId int ,
        MemberName varchar (256) ,
        CategoryName varchar (50) 
);

CREATE FUNCTION GetAdById (pId int) RETURNS SETOF GetAdByIdType AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT     classifieds_Ads.Id, classifieds_Ads.MemberId, classifieds_Ads.CategoryId, classifieds_Ads.Title, classifieds_Ads.Description, classifieds_Ads.URL, classifieds_Ads.Price, classifieds_Ads.Location, 
                      classifieds_Ads.ExpirationDate, classifieds_Ads.DateCreated, classifieds_Ads.DateApproved, classifieds_Ads.NumViews, classifieds_Ads.NumResponses, classifieds_Ads.AdLevel, 
                      classifieds_Ads.AdStatus, classifieds_Ads.AdType, classifieds_Ads.PreviewImageId, classifieds_Members.AspNetUsername AS MemberName, 
                      classifieds_Categories.Name AS CategoryName
FROM         classifieds_Ads INNER JOIN
                      classifieds_Members ON classifieds_Ads.MemberId = classifieds_Members.Id INNER JOIN
                      classifieds_Categories ON classifieds_Ads.CategoryId = classifieds_Categories.Id 
WHERE classifieds_Ads.Id = pId LOOP
      	RETURN NEXT r;
   END LOOP;
   RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION GetAdsByRandomOrder (pNumRecords int, pAdStatus int, pAdLevel int) RETURNS SETOF ClassifiedsView_Ads AS $$
DECLARE
	r RECORD;
BEGIN
    FOR r IN SELECT * FROM ClassifiedsView_Ads 
    WHERE AdStatus = pAdStatus AND AdLevel = pAdLevel And PreviewImageId IS NOT NULL LOOP
	RETURN NEXT r;
    END LOOP;
    RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION GetAdsByStatus (pAdStatus int, pMemberId int) RETURNS SETOF ClassifiedsView_Ads AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT * FROM ClassifiedsView_Ads
	WHERE
		AdStatus = pAdStatus AND
		(pMemberId = 0 OR MemberId = pMemberId) LOOP
	     RETURN NEXT r;
	END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION GetAllAdsByQuery (
       pLimitResultCount int,
       pCategoryId int,
       pMemberId int,
       pMaxPrice NUMERIC(19,4),
       pSearchTerm varchar(50),
       pLocation varchar(50),
       pAdType int,
       pAdStatus int,
       pAdLevel int,
       pMinDateCreated timestamptz,
       pMustHaveImage boolean) RETURNS SETOF ClassifiedsView_Ads AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT *
	FROM ClassifiedsView_Ads
	WHERE
	(pCategoryId = 0 OR CategoryId IN (
	SELECT Id FROM classifieds_Categories
	WHERE Path LIKE
	  (SELECT Path
	   FROM classifieds_Categories
	   WHERE Id = pCategoryId ) || '%'
	)) AND 
	(pMaxPrice = -1 OR Price <= pMaxPrice) AND
	(pMustHaveImage = '0' OR PreviewImageId IS NOT NULL) AND
	(pAdStatus = 0 OR AdStatus = pAdStatus) AND
	(pAdLevel = 0 OR AdLevel = pAdLevel) AND
	(pAdType = 0 OR AdType = pAdType) AND 
	(pMemberId = 0 OR MemberId = pMemberId) AND 
	(pMinDateCreated IS NULL OR DateCreated > pMinDateCreated) AND
	Title LIKE '%' || pSearchTerm || '%' AND 
	Location LIKE '%' || pLocation || '%'
    ORDER BY DateCreated DESC LIMIT pLimitResultCount LOOP
         RETURN NEXT r;
    END LOOP;
    RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION GetAllLocations () RETURNS SETOF classifieds_Locations AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT     *
	FROM         classifieds_Locations ORDER BY Name LOOP
	    RETURN NEXT r;
        END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;


CREATE TYPE GetCategoriesSubType AS (
       Id integer,
       ParentCategoryId integer,
       Name varchar (50),
       NumActiveAds integer
);
CREATE FUNCTION GetCategoriesByParentId (pParentCategoryId int) RETURNS SETOF GetCategoriesSubType AS $$
DECLARE
	r RECORD;
	rowCount int;
BEGIN
	FOR r IN SELECT Id, ParentCategoryId, Name, NumActiveAds
	FROM classifieds_Categories WHERE (ParentCategoryId = pParentCategoryId)
		 OR (pParentCategoryId = 0 AND ParentCategoryId IS NULL) ORDER BY Name LOOP
	      RETURN NEXT r;
	END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION GetCategoryById (pId int) RETURNS SETOF classifieds_Categories AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT * FROM classifieds_Categories WHERE Id = pId LOOP
	     RETURN NEXT r;
        END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION GetExpiredAds (pExpirationDate timestamptz, pAdStatus int) RETURNS SETOF ClassifiedsView_Ads AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT * FROM ClassifiedsView_Ads 
	WHERE
		ExpirationDate < pExpirationDate AND
		(pAdStatus = 0 OR AdStatus = pAdStatus) LOOP
	    RETURN NEXT r;
	END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION GetMemberByUsername (pAspNetUsername varchar(256), pAspNetApplicationName varchar(256)) 
RETURNS SETOF classifieds_Members AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT     *
	FROM         classifieds_Members 
	WHERE     (AspNetUsername = pAspNetUsername) AND (AspNetApplicationName = pAspNetApplicationName) LOOP
	     RETURN NEXT r;
	END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION GetParentCategoriesById (pId int) RETURNS SETOF classifieds_Categories AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT *
	FROM classifieds_Categories 
	WHERE (SELECT Path
       	      FROM classifieds_Categories
        WHERE Id = pId) LIKE Path || '%' ORDER BY Path LOOP
	    RETURN NEXT r;
	END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION GetPhotoById (pId int, pSize int) RETURNS SETOF PhotosView AS $$
DECLARE
	r RECORD;
BEGIN
IF pSize = 1 THEN
	FOR r IN SELECT     	*
	FROM         PhotosView 
	WHERE Id = pId LOOP
	      RETURN NEXT r;
	END LOOP;
ELSE
	FOR r IN SELECT     	*
	FROM         PhotosView 
	WHERE Id = pId LOOP
	      RETURN NEXT r;
	END LOOP;
END IF;
	RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE TYPE GetPhotoBytesByIdType AS (
	Bytes bytea 
);


CREATE FUNCTION GetPhotoBytesById (pId int, pSize int) RETURNS SETOF GetPhotoBytesByIdType AS $$
DECLARE
	r RECORD;
BEGIN
IF pSize = 3 THEN
	FOR r IN SELECT     	BytesFull AS Bytes
	FROM         classifieds_Photos 
	WHERE Id = pId LOOP
	     RETURN NEXT r;
	END LOOP;
ELSIF pSize = 2 THEN
	FOR r IN SELECT     	BytesMedium AS Bytes
	FROM         classifieds_Photos 
	WHERE Id = pId LOOP
	     RETURN NEXT r;
	END LOOP;
ELSE
	FOR r IN SELECT     	BytesSmall AS Bytes
	FROM         classifieds_Photos 
	WHERE Id = pId LOOP
	     RETURN NEXT r;
	END LOOP;
END IF;
	RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION GetPhotosByAdId (pAdId int) RETURNS SETOF classifieds_Photos AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT     *
	FROM         classifieds_Photos 
	WHERE     (AdId = pAdId)
	ORDER BY IsMainPreview DESC, DateCreated LOOP
	      RETURN NEXT r;
	END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION GetPhotosByAdStatus (pAdStatus int) RETURNS SETOF classifieds_Photos AS $$
DECLARE 
	pNullBytes bytea;
	r RECORD;
BEGIN
	pNullBytes = NULL;

	FOR r IN SELECT     classifieds_Photos.Id, classifieds_Photos.AdId, classifieds_Photos.IsMainPreview, 
                      classifieds_Photos.DateCreated
		      FROM         classifieds_Photos INNER JOIN
                      classifieds_Ads ON classifieds_Photos.Id = classifieds_Ads.Id
		      WHERE     (classifieds_Ads.AdStatus = pAdStatus) LOOP
	      RETURN NEXT r;
	END LOOP;
	RETURN;
END;
$$ LANGUAGE plpgsql;


CREATE FUNCTION GetSavedAds (pMemberId int) RETURNS SETOF GetAdByIdType AS $$
DECLARE
	r RECORD;
BEGIN
	FOR r IN SELECT     classifieds_Ads.Id, classifieds_Ads.MemberId, classifieds_Ads.CategoryId, classifieds_Ads.Title, classifieds_Ads.Description, classifieds_Ads.URL, classifieds_Ads.Price, classifieds_Ads.Location, 
                      classifieds_Ads.ExpirationDate, classifieds_Ads.DateCreated, classifieds_Ads.DateApproved, classifieds_Ads.NumViews, classifieds_Ads.NumResponses, classifieds_Ads.AdLevel, 
                      classifieds_Ads.AdStatus, classifieds_Ads.AdType, classifieds_Ads.PreviewImageId, classifieds_Members.AspNetUsername AS MemberName, 
                      classifieds_Categories.Name AS CategoryName
FROM         classifieds_Ads INNER JOIN
                      classifieds_SavedAds ON classifieds_Ads.Id = classifieds_SavedAds.AdId INNER JOIN
                      classifieds_Members ON classifieds_Ads.MemberId = classifieds_Members.Id INNER JOIN
                      classifieds_Categories ON classifieds_Ads.CategoryId = classifieds_Categories.Id
WHERE classifieds_SavedAds.MemberId = pMemberId
ORDER BY classifieds_SavedAds.DateCreated DESC LOOP
     RETURN NEXT r;
END LOOP;
RETURN;
END;
$$ LANGUAGE plpgsql;


CREATE FUNCTION InsertAd
(
	pMemberId int,
	pCategoryId int,
	pTitle varchar(100),
	pDescription text,
	pURL varchar(500),
	pPrice NUMERIC(19,4),
	pLocation varchar(50),
	pExpirationDate timestamptz,
	pDateCreated timestamptz,
	pDateApproved timestamptz,
	pNumViews int,
	pNumResponses int,
	pAdLevel int,
	pAdStatus int,
	pAdType int
) RETURNS SETOF IdReturn AS $$
DECLARE 
	pId int;
	r RECORD;
BEGIN
INSERT INTO classifieds_Ads
                      (MemberId, CategoryId, Title, Description, URL, Price, Location, ExpirationDate, DateCreated, DateApproved, NumViews, NumResponses, AdLevel, 
                      AdStatus, AdType)
VALUES     (pMemberId,pCategoryId,pTitle,pDescription,pURL,pPrice,pLocation,pExpirationDate,pDateCreated,pDateApproved,pNumViews,pNumResponses,pAdLevel,pAdStatus,pAdType);
pId = currval('classifieds_ads_id_seq');
FOR r IN SELECT pId AS Id LOOP
    RETURN NEXT r;
END LOOP;
RETURN;
END;
$$ LANGUAGE plpgsql;


CREATE FUNCTION InsertCategory (pParentCategoryId int, pName varchar(50)) RETURNS SETOF IdReturn AS $$
DECLARE 
	pId int;
	r RECORD;
BEGIN
INSERT INTO classifieds_Categories
                      (ParentCategoryId, Name)
VALUES     (pParentCategoryId,pName);
pId = currval('classifieds_categories_id_seq');
FOR r IN SELECT pId AS Id LOOP
    RETURN NEXT r;
END LOOP;
RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION InsertLocation (pName varchar(50)) RETURNS void AS $$
BEGIN
	INSERT INTO classifieds_Locations (Name) VALUES (pName);
END;
$$ LANGUAGE plpgsql;


CREATE FUNCTION InsertMember (
	pAspNetUsername varchar(256),
	pAspNetApplicationName varchar(256),
	pDateCreated timestamptz) RETURNS SETOF IdReturn AS $$
DECLARE 
	pId int;
	r RECORD;
BEGIN
INSERT INTO classifieds_Members (AspNetUsername, AspNetApplicationName, DateCreated) VALUES (pAspNetUsername, pAspNetApplicationName, pDateCreated);
pId = currval('classifieds_members_id_seq');
FOR r IN SELECT  pId  AS Id LOOP
    RETURN NEXT r;
END LOOP;
RETURN;
END;
$$ LANGUAGE plpgsql;


CREATE FUNCTION InsertPhoto (
       pAdId int,
       pBytesFull bytea,
       pBytesMedium bytea,
       pBytesSmall bytea,
       pIsMainPreview boolean,
       pDateCreated timestamptz) RETURNS SETOF IdReturn AS $$
DECLARE 
	pId int;
	r RECORD;
BEGIN
INSERT INTO classifieds_Photos
(AdId, BytesFull, BytesMedium, BytesSmall, IsMainPreview, DateCreated)
VALUES
(pAdId, pBytesFull, pBytesMedium, pBytesSmall, pIsMainPreview, pDateCreated);
pId = currval('classifieds_photos_id_seq');

IF pIsMainPreview = TRUE THEN
   UPDATE classifieds_Ads SET PreviewImageId = pId WHERE Id = pAdId;
END IF;

FOR r IN SELECT pId AS Id LOOP
    RETURN NEXT r;
END LOOP;
RETURN;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION InsertSavedAd (pAdId int, pMemberId int) RETURNS void AS $$
DECLARE
	rowCount int;
BEGIN
SELECT INTO rowCount 1 FROM classifieds_SavedAds WHERE AdId = pAdId AND MemberId = pMemberId limit 1;
IF NOT found THEN
INSERT INTO classifieds_SavedAds
	(AdId, MemberId, DateCreated)
VALUES
	(pAdId, pMemberId, now());
END IF;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION MoveAds (pCurrentCategoryId int, pNewCategoryId int) RETURNS void AS $$
BEGIN
UPDATE classifieds_Ads
SET
	CategoryId = pNewCategoryId
WHERE
	CategoryId = pCurrentCategoryId;
END;
$$ LANGUAGE plpgsql;


CREATE FUNCTION MoveCategory (pCategoryId int, pNewParentCategoryId int) RETURNS int AS $$
DECLARE 
	pNumActiveAdsInCategory int;
	pOldPath varchar(800);
	pNewPath varchar(800);
	pNegativeCount int;
BEGIN
-- Stored Procedure will not execute, if:

-- 1st case: Current Category = New Parent Category (it cannot become a parent of itself)
IF pCategoryId = pNewParentCategoryId THEN
	RETURN -1;
END IF;

-- 2nd case: the input category is a parent of the new category
-- (a parent category cannot become a child category under one of its own children)
IF EXISTS (SELECT Id FROM classifieds_Categories
		WHERE
			(SELECT Path
			FROM classifieds_Categories
			WHERE Id = pCategoryId) LIKE Path || '%'
		AND
			Id = pNewParentCategoryId) THEN
	RETURN -1; -- exits
END IF;
SELECT ParentCategories.Path

FROM         classifieds_Categories INNER JOIN INTO pOldPath
                      classifieds_Categories ParentCategories ON classifieds_Categories.ParentCategoryId = ParentCategories.Id
WHERE     (classifieds_Categories.Id = pCategoryId);

SELECT CASE WHEN
		pNewParentCategoryId IS NULL THEN '.'
	ELSE 
		Path
	END
	FROM classifieds_Categories INTO pNewPath
	WHERE pNewParentCategoryId is NULL OR Id = pNewParentCategoryId;

IF pOldPath IS NULL THEN
	SELECT Path FROM classifieds_Categories INTO pOldPath WHERE Id = pCategoryId;
	pNewPath = pNewPath || CAST(pCategoryId AS VARCHAR(10)) || '.';
END IF;

SELECT Count(Id) FROM ClassifiedsView_Ads INTO pNumActiveAdsInCategory WHERE 
AdStatus >= 100 AND 
ExpirationDate > now() AND
ClassifiedsView_Ads.CategoryId IN (SELECT Id FROM classifieds_Categories
WHERE Path LIKE
  (SELECT Path
   FROM classifieds_Categories
   WHERE Id = pCategoryId ) || '%' );

pNegativeCount = 0 - pNumActiveAdsInCategory;
PERFORM UpdateCategoryAdCounts (pCategoryId, pNegativeCount);

UPDATE classifieds_Categories SET Path = REPLACE(Path, pOldPath, pNewPath)

WHERE Path LIKE

  (SELECT Path
   FROM classifieds_Categories
   WHERE Id = pCategoryId ) || '%';

PERFORM UpdateCategoryAdCounts (pCategoryId, pNumActiveAdsInCategory);

UPDATE classifieds_Categories SET ParentCategoryId = pNewParentCategoryId WHERE Id = pCategoryId;

RETURN 1;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION RelistAd (
	pAdId int,
	pCategoryId int,
	pTitle varchar(100),
	pDescription text,
	pURL varchar(500),
	pPrice NUMERIC(19,4),
	pLocation varchar(50),
	pExpirationDate timestamptz,
	pDateCreated timestamptz,
	pDateApproved timestamptz,
	pAdLevel int,
	pAdStatus int,
	pAdType int) RETURNS void AS $$
BEGIN
UPDATE classifieds_Ads
SET
	CategoryId = pCategoryId,
	Title = pTitle ,
	Description = pDescription ,
	URL = pURL,
	Price = pPrice, 
	Location = pLocation, 
	ExpirationDate = pExpirationDate ,
	-- @DateCreated timestamptz = NULL,
	DateApproved = pDateApproved, 
	AdLevel = pAdLevel, 
	AdStatus = pAdStatus, 
	AdType = pAdType,
	NumResponses = 0,
	NumViews = 0	
WHERE Id = pAdId;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION RemoveAd (pAdId int) RETURNS void AS $$
BEGIN
	DELETE FROM classifieds_Photos WHERE AdId = pAdId;
	DELETE FROM classifieds_SavedAds WHERE AdId = pAdId;
	DELETE FROM classifieds_Ads WHERE Id = pAdId;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION RemoveAdsByStatus (pAdStatus int) RETURNS void AS $$
BEGIN
      DELETE FROM classifieds_Photos WHERE AdId IN (SELECT Id FROM classifieds_Ads WHERE AdStatus = pAdStatus);
      DELETE FROM classifieds_SavedAds WHERE AdId IN (SELECT Id FROM classifieds_Ads WHERE AdStatus = pAdStatus);
      DELETE FROM classifieds_Ads WHERE AdStatus = pAdStatus;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION RemoveCategory (pId int) RETURNS int AS $$
DECLARE 
	returnval int;
BEGIN
returnval = 1;

-- cannot remove a category with ads in it (they must be moved first)
---- return -1
IF EXISTS(SELECT Id FROM classifieds_Ads WHERE
			(classifieds_Ads.CategoryId IN 
				(SELECT Id FROM classifieds_Categories
				WHERE Path LIKE
					(SELECT Path
					FROM classifieds_Categories
					WHERE Id = pId ) || '%'))) THEN
	returnval = -1;
END IF;
-- cannot remove category if it is a parent of other Categories
---- return -2
IF EXISTS(SELECT Id FROM classifieds_Categories WHERE ParentCategoryId = pId) THEN
    returnval = -2;
END IF;

-- can remove category
IF (returnval = 1) THEN
  DELETE FROM classifieds_Categories WHERE Id = pId;
END IF;
RETURN returnval;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION RemoveLocation (pId int) RETURNS void AS $$
BEGIN
	DELETE FROM classifieds_Locations WHERE Id = pId;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION RemovePhoto (pId int) RETURNS void AS $$
DECLARE 
	pAdId int;
BEGIN
	SELECT into pAdId AdId
	FROM classifieds_Photos
	WHERE Id = pId;

	DELETE FROM classifieds_Photos WHERE Id = pId;

IF NOT EXISTS(SELECT Id FROM classifieds_Photos WHERE AdId = pAdId AND IsMainPreview = TRUE) THEN
	UPDATE
		classifieds_Photos SET IsMainPreview = TRUE
		WHERE Id =
		(SELECT Id FROM classifieds_Photos WHERE AdId = pAdId LIMIT 1);
	UPDATE    
		classifieds_Ads SET PreviewImageId =
        (SELECT Id
         FROM classifieds_Photos
         WHERE (AdId = pAdId) AND (IsMainPreview = TRUE)
         )
		WHERE (Id = pAdId);
END IF;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION RemovePhotosByAdStatus (pAdStatus int) RETURNS void AS $$
BEGIN
	UPDATE classifieds_Ads SET PreviewImageId = NULL WHERE AdStatus = pAdStatus;
	DELETE FROM classifieds_Photos
	WHERE AdId IN
	      (SELECT Id FROM classifieds_Ads WHERE AdStatus = pAdStatus);
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION RemoveSavedAd (pAdId int, pMemberId int) RETURNS void AS $$
BEGIN
	DELETE FROM classifieds_SavedAds WHERE AdId = pAdId AND MemberId = pMemberId;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION SetAdPreviewPhoto (pPhotoId int, pAdId int) RETURNS void AS $$
BEGIN
UPDATE classifieds_Photos SET
	IsMainPreview = CASE WHEN pPhotoId=Id THEN TRUE ELSE FALSE END
WHERE AdId = pAdId;

UPDATE classifieds_Ads
	SET PreviewImageId = pPhotoId
WHERE
	Id = pAdId;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION UpdateAd (
	pId int,
	pMemberId int,
	pTitle varchar(100),
	pDescription text,
	pURL varchar(500),
	pPrice NUMERIC(19,4),
	pLocation varchar(50)) RETURNS void AS $$
BEGIN
UPDATE classifieds_Ads
SET
	Title = pTitle,
	Description = pDescription,
	Url = pURL,
	Price = pPrice,
	Location = pLocation
WHERE
	Id = pId;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION UpdateAdCategory (pId int, pCategoryId int) RETURNS void AS $$
BEGIN
UPDATE classifieds_Ads
SET 
	CategoryId = pCategoryId
WHERE
	Id = pId;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION UpdateAdLevel (pId int, pAdLevel int) RETURNS void AS $$
BEGIN
UPDATE classifieds_Ads
SET 
	AdLevel = pAdLevel
WHERE
	Id = pId;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION UpdateAdStats (pAdId int, pNumberToAddToViews int, pNumberToAddToResponses int) RETURNS void AS $$
BEGIN
UPDATE classifieds_Ads SET
	NumViews = NumViews + pNumberToAddToViews,
	NumResponses = NumResponses + pNumberToAddToResponses
WHERE
	Id = pAdId;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION UpdateAdStatus (pId int, pAdStatus int) RETURNS void AS $$
BEGIN
UPDATE classifieds_Ads
SET 
	AdStatus = pAdStatus
WHERE
	Id = pId;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION UpdateCategory
(
	pId int,
	pParentCategoryId int,
	pName varchar(50),
	pNumActiveAds int
) RETURNS void AS $$
BEGIN
UPDATE classifieds_Categories SET ParentCategoryId = pParentCategoryId,  Name = pName, NumActiveAds = pNumActiveAds WHERE ((Id = pId));
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION UpdateCategoryAdCounts (pLeafCategoryId int, pNumberToAdd int) RETURNS void AS $$
BEGIN
UPDATE classifieds_Categories SET NumActiveAds = NumActiveAds + pNumberToAdd
WHERE Id IN
	(SELECT Id
	FROM classifieds_Categories
	WHERE (SELECT Path
	       FROM classifieds_Categories
	       WHERE Id = pLeafCategoryId) LIKE Path || '%');
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION UpdateCategoryName (pId int, pName varchar(50)) RETURNS void AS $$
BEGIN
UPDATE classifieds_Categories
SET Name = pName
WHERE Id = pId;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION UpdateLocation (pId int, pName varchar(50)) RETURNS void AS $$
BEGIN
UPDATE classifieds_Locations
SET Name = pName
WHERE Id = pId;
END;
$$ LANGUAGE plpgsql;



CREATE FUNCTION AdDeletedTrigger () RETURNS trigger AS $$
BEGIN
  IF OLD.AdStatus >= 100 THEN
    -- decrement the affected category counts by subtracting 1
    PERFORM UpdateCategoryAdCounts (OLD.CategoryId, -1);
  END IF;
  RETURN OLD;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER AdDeleted AFTER DELETE ON classifieds_Ads FOR EACH ROW EXECUTE PROCEDURE AdDeletedTrigger();

CREATE OR REPLACE FUNCTION AdInsertedTrigger () RETURNS trigger AS $$
BEGIN
  IF NEW.AdStatus >= 100 THEN
    -- increment the affected category counts by adding +1
    PERFORM UpdateCategoryAdCounts (NEW.CategoryId, 1);
  END IF;
  RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER AdInserted AFTER INSERT ON classifieds_Ads FOR EACH ROW EXECUTE PROCEDURE AdInsertedTrigger ();


CREATE FUNCTION AdUpdatedTrigger () RETURNS trigger AS $$
BEGIN
  IF (NEW.CategoryId = OLD.CategoryId) THEN
    IF (NEW.AdStatus < 100 AND OLD.AdStatus >= 100) THEN
      -- decrement the affected category counts by subtracting 1
       PERFORM UpdateCategoryAdCounts (OLD.CategoryId, -1);
    ELSIF (NEW.AdStatus >= 100 AND OLD.AdStatus < 100) THEN
      -- increment the affected category counts by adding +1
      PERFORM UpdateCategoryAdCounts (NEW.CategoryId, 1);
    END IF;
  ELSE
    IF OLD.AdStatus >= 100 THEN
      -- decrement the affected category counts by subtracting 1
       PERFORM UpdateCategoryAdCounts (OLD.CategoryId, -1);
    END IF;
    IF NEW.AdStatus >= 100 THEN
      -- increment the affected category counts by adding +1
      PERFORM UpdateCategoryAdCounts (NEW.CategoryId, 1);
    END IF;    
  END IF;
  RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER AdUpdated AFTER UPDATE ON classifieds_Ads FOR EACH ROW EXECUTE PROCEDURE AdUpdatedTrigger ();


CREATE FUNCTION OnCategoryInsertTrigger () RETURNS trigger AS $$
BEGIN
UPDATE classifieds_Categories
SET    
    Path = 
      CASE
          WHEN NEW.ParentCategoryId IS NULL THEN '.' 
          ELSE ParentCategory.Path
          END || CAST(NEW.Id AS varchar(10)) || '.'
FROM        classifieds_Categories Category
LEFT OUTER JOIN
            classifieds_Categories ParentCategory ON Category.ParentCategoryId = ParentCategory.Id
WHERE Category.Id = NEW.Id AND classifieds_Categories.Id = New.Id;
RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER OnCategoryInsert AFTER INSERT ON classifieds_Categories FOR EACH ROW EXECUTE PROCEDURE OnCategoryInsertTrigger ();

-- alter object owners
ALTER TABLE Users OWNER TO test;
ALTER INDEX users_email_index OWNER TO test;
ALTER INDEX users_islockedout_index OWNER TO test;
ALTER TABLE Roles OWNER TO test;
ALTER TABLE UsersInRoles OWNER TO test;
ALTER TABLE Profiles OWNER TO test;
ALTER INDEX profiles_isanonymous_index OWNER TO test;
ALTER TABLE ProfileData OWNER TO test;
ALTER TABLE classifieds_Ads OWNER TO test;
ALTER TABLE classifieds_Categories OWNER TO test;
ALTER TABLE classifieds_Locations OWNER TO test;
ALTER TABLE classifieds_Members OWNER TO test;
ALTER TABLE classifieds_Photos OWNER TO test;
ALTER TABLE classifieds_SavedAds OWNER TO test;
ALTER TABLE ClassifiedsView_Ads OWNER TO test;
ALTER TABLE PhotosView OWNER TO test;
ALTER TYPE CountReturn OWNER TO test;
ALTER FUNCTION CountAdResponsesByStatus (pAdStatus int) OWNER TO test;
ALTER FUNCTION CountAdViewsByStatus (pAdStatus int) OWNER TO test;
ALTER TYPE IdReturn OWNER TO test;
ALTER TYPE AdCountReturn OWNER TO test;
ALTER FUNCTION CountAdsByCategory (pCategoryId int, pAdStatus int) OWNER TO test;
ALTER FUNCTION CountAdsByStatus (pAdStatus int, pMinDateCreated timestamptz) OWNER TO test;
ALTER FUNCTION CountLocations () OWNER TO test;
ALTER FUNCTION CountMembersByDateRange (pStartDate timestamptz, pEndDate timestamptz) OWNER TO test;
ALTER FUNCTION CountTopCategories () OWNER TO test;
ALTER FUNCTION CreateCategory (pParentCategoryId int, pName varchar(50)) OWNER TO test;
ALTER FUNCTION ExpireAd (pAdId int, pMemberId int, pAdStatus int) OWNER TO test;
ALTER TYPE GetAdByIdType OWNER TO test;
ALTER FUNCTION GetAdById (pId int) OWNER TO test;
ALTER FUNCTION GetAdsByRandomOrder (pNumRecords int, pAdStatus int, pAdLevel int) OWNER TO test;
ALTER FUNCTION GetAdsByStatus (pAdStatus int, pMemberId int) OWNER TO test;
ALTER FUNCTION GetAllAdsByQuery (
       pLimitResultCount int,
       pCategoryId int,
       pMemberId int,
       pMaxPrice NUMERIC(19,4),
       pSearchTerm varchar(50),
       pLocation varchar(50),
       pAdType int,
       pAdStatus int,
       pAdLevel int,
       pMinDateCreated timestamptz,
       pMustHaveImage boolean)  OWNER TO test;
ALTER FUNCTION GetAllLocations () OWNER TO test;
ALTER TYPE GetCategoriesSubType OWNER TO test;
ALTER FUNCTION GetCategoriesByParentId (pParentCategoryId int) OWNER TO test;
ALTER FUNCTION GetCategoryById (pId int) OWNER TO test;
ALTER FUNCTION GetExpiredAds (pExpirationDate timestamptz, pAdStatus int) OWNER TO test;
ALTER FUNCTION GetMemberByUsername (pAspNetUsername varchar(256), pAspNetApplicationName varchar(256)) 
 OWNER TO test;
ALTER FUNCTION GetParentCategoriesById (pId int) OWNER TO test;
ALTER FUNCTION GetPhotoById (pId int, pSize int) OWNER TO test;
ALTER TYPE GetPhotoBytesByIdType OWNER TO test;
ALTER FUNCTION GetPhotoBytesById (pId int, pSize int) OWNER TO test;
ALTER FUNCTION GetPhotosByAdId (pAdId int) OWNER TO test;
ALTER FUNCTION GetPhotosByAdStatus (pAdStatus int) OWNER TO test;
ALTER FUNCTION GetSavedAds (pMemberId int) OWNER TO test;
-- TODO: create a type to return
ALTER FUNCTION InsertAd
(
	pMemberId int,
	pCategoryId int,
	pTitle varchar(100),
	pDescription text,
	pURL varchar(500),
	pPrice NUMERIC(19,4),
	pLocation varchar(50),
	pExpirationDate timestamptz,
	pDateCreated timestamptz,
	pDateApproved timestamptz,
	pNumViews int,
	pNumResponses int,
	pAdLevel int,
	pAdStatus int,
	pAdType int
) OWNER TO test;
ALTER FUNCTION InsertCategory (pParentCategoryId int, pName varchar(50)) OWNER TO test;
ALTER FUNCTION InsertLocation (pName varchar(50)) OWNER TO test;
ALTER FUNCTION InsertMember (
	pAspNetUsername varchar(256),
	pAspNetApplicationName varchar(256),
	pDateCreated timestamptz) OWNER TO test;
ALTER FUNCTION InsertPhoto (
       pAdId int,
       pBytesFull bytea,
       pBytesMedium bytea,
       pBytesSmall bytea,
       pIsMainPreview boolean,
       pDateCreated timestamptz) OWNER TO test;
ALTER FUNCTION InsertSavedAd (pAdId int, pMemberId int) OWNER TO test;
ALTER FUNCTION MoveAds (pCurrentCategoryId int, pNewCategoryId int) OWNER TO test;
ALTER FUNCTION MoveCategory (pCategoryId int, pNewParentCategoryId int) OWNER TO test;
ALTER FUNCTION RelistAd (
	pAdId int,
	pCategoryId int,
	pTitle varchar(100),
	pDescription text,
	pURL varchar(500),
	pPrice NUMERIC(19,4),
	pLocation varchar(50),
	pExpirationDate timestamptz,
	pDateCreated timestamptz,
	pDateApproved timestamptz,
	pAdLevel int,
	pAdStatus int,
	pAdType int) OWNER TO test;
ALTER FUNCTION RemoveAd (pAdId int) OWNER TO test;
ALTER FUNCTION RemoveAdsByStatus (pAdStatus int) OWNER TO test;
ALTER FUNCTION RemoveCategory (pId int) OWNER TO test;
ALTER FUNCTION RemoveLocation (pId int) OWNER TO test;
ALTER FUNCTION RemovePhoto (pId int) OWNER TO test;
ALTER FUNCTION RemovePhotosByAdStatus (pAdStatus int) OWNER TO test;
ALTER FUNCTION RemoveSavedAd (pAdId int, pMemberId int) OWNER TO test;
ALTER FUNCTION SetAdPreviewPhoto (pPhotoId int, pAdId int) OWNER TO test;
ALTER FUNCTION UpdateAd (
	pId int,
	pMemberId int,
	pTitle varchar(100),
	pDescription text,
	pURL varchar(500),
	pPrice NUMERIC(19,4),
	pLocation varchar(50)) OWNER TO test;
ALTER FUNCTION UpdateAdCategory (pId int, pCategoryId int) OWNER TO test;
ALTER FUNCTION UpdateAdLevel (pId int, pAdLevel int) OWNER TO test;
ALTER FUNCTION UpdateAdStats (pAdId int, pNumberToAddToViews int, pNumberToAddToResponses int) OWNER TO test;
ALTER FUNCTION UpdateAdStatus (pId int, pAdStatus int) OWNER TO test;
ALTER FUNCTION UpdateCategory
(
	pId int,
	pParentCategoryId int,
	pName varchar(50),
	pNumActiveAds int
) OWNER TO test;
ALTER FUNCTION UpdateCategoryAdCounts (pLeafCategoryId int, pNumberToAdd int) OWNER TO test;
ALTER FUNCTION UpdateCategoryName (pId int, pName varchar(50)) OWNER TO test;
ALTER FUNCTION UpdateLocation (pId int, pName varchar(50)) OWNER TO test;
ALTER FUNCTION AdDeletedTrigger () OWNER TO test;
ALTER FUNCTION AdInsertedTrigger () OWNER TO test;
ALTER FUNCTION AdUpdatedTrigger () OWNER TO test;
ALTER FUNCTION OnCategoryInsertTrigger () OWNER TO test;

-- test user data
INSERT INTO classifieds_members (id, aspnetusername, aspnetapplicationname, datecreated) VALUES (1, 'test', '/', '2007-05-23 10:43:21.7331+02');

INSERT INTO roles (rolename, applicationname) VALUES ('Administrators', '/');
INSERT INTO roles (rolename, applicationname) VALUES ('Guests', '/');

INSERT INTO users (pid, username, applicationname, email, "comment", "password", passwordquestion, passwordanswer, isapproved, lastactivitydate, lastlogindate, lastpasswordchangeddate, creationdate, isonline, islockedout, lastlockedoutdate, failedpasswordattemptcount, failedpasswordattemptwindowstart, failedpasswordanswerattemptcount, failedpasswordanswerattemptwindowstart) VALUES ('8bd91591-9d68-42ba-9f1b-1ef42f215e35', 'test', '/', 'test@example.com', '', 'YX6V08gSdP+RYgRtaVePad6wdVw=', 'question', 'dnc4cGDJy7KCmNn7vJJHG5OCbdU=', true, '2007-05-23 10:43:21.694593+02', '2007-05-23 10:43:21.773929+02', '2007-05-23 10:43:21.694593+02', '2007-05-23 10:43:21.694593+02', NULL, false, '2007-05-23 10:43:21.694593+02', 0, '2007-05-23 10:43:21.694593+02', 0, '2007-05-23 10:43:21.694593+02');

INSERT INTO usersinroles (username, rolename, applicationname) VALUES ('test', 'Administrators', '/');

INSERT INTO profiles (pid, username, applicationname, isanonymous, lastactivitydate, lastupdateddate) VALUES ('dc758e19-a052-45a8-901f-61606ade7a65', 'test', '/', false, '2007-05-23 10:43:21.758005+02', '2007-05-23 10:43:21.758021+02');

INSERT INTO profiledata (pid, profile, name, valuestring, valuebinary) VALUES ('a00a0695-e591-4e33-9530-3ce1236a4db2', 'dc758e19-a052-45a8-901f-61606ade7a65', 'LastName', 'AAEAAAD/////AQAAAAAAAAAGAQAAAARVc2VyCw==', NULL);
INSERT INTO profiledata (pid, profile, name, valuestring, valuebinary) VALUES ('2d0c6ae0-b6b4-4e5e-8c51-c818398053e7', 'dc758e19-a052-45a8-901f-61606ade7a65', 'FirstName', 'AAEAAAD/////AQAAAAAAAAAGAQAAAARUZXN0Cw==', NULL);
INSERT INTO profiledata (pid, profile, name, valuestring, valuebinary) VALUES ('be81330b-ae0e-491c-b827-1efddeb8c120', 'dc758e19-a052-45a8-901f-61606ade7a65', 'MemberId', 'AAEAAAD/////AQAAAAAAAAAEAQAAAAxTeXN0ZW0uSW50MzIBAAAAB21fdmFsdWUACAEAAAAL', NULL);

